@@ -9,9 +9,8 @@ from mypy.test.helpers import (
 )
 from mypy.test.data import DataDrivenTestCase, DataSuite
 from mypy.test.config import test_temp_dir
+from mypy.test.visitors import TypeAssertTransformVisitor
 from mypy.errors import CompileError
-from mypy.treetransform import TransformVisitor
-from mypy.types import Type
 from mypy.options import Options
 
 
@@ -72,9 +71,3 @@ def test_transform(testcase: DataDrivenTestCase) -> None:
         testcase.output, a,
         'Invalid semantic analyzer output ({}, line {})'.format(testcase.file,
                                                                 testcase.line))
-
-
-class TypeAssertTransformVisitor(TransformVisitor):
-    def type(self, type: Type) -> Type:
-        assert type is not None
-        return type
